searchNodes=[{"doc":"Performs RPC calls to nodes in Fly.io regions. Provides features to help Elixir applications more easily take advantage of the features that Fly.io provides. Configuration Assumes each node is running the Fly.RPC server in its supervision tree and exports FLY_REGION environment variable to identify the fly region. Note : anonymous function support only works when the release is identical across all nodes. This can be ensured by including the FLY_IMAGE_REF as part of the node name in your rel/env.sh.eex file: #!/bin/sh export ERL_AFLAGS = &quot;-proto_dist inet6_tcp&quot; export RELEASE_DISTRIBUTION = name export RELEASE_NODE = &quot;${FLY_APP_NAME}-${FLY_IMAGE_REF##*-}@${FLY_PRIVATE_IP}&quot; To run code on a specific region call rpc_region/4 . A node found within the given region will be chosen at random. Raises if no nodes exist on the given region. The special :primary region may be passed to run the rpc against the region identified by the PRIMARY_REGION environment variable. Examples &gt; rpc_region ( &quot;hkg&quot; , fn -&gt; String . upcase ( &quot;fly&quot; ) end ) &quot;FLY&quot; &gt; rpc_region ( &quot;hkg&quot; , { String , :upcase , [ &quot;fly&quot; ] } ) &quot;FLY&quot; &gt; rpc_region ( Fly.RPC . primary_region ( ) , { String , :upcase , [ &quot;fly&quot; ] ] ) &quot;FLY&quot; &gt; rpc_region ( :primary , { String , :upcase , [ &quot;fly&quot; ] } ) &quot;FLY&quot; Server The GenServer's responsibility is just to monitor other nodes as they enter and leave the cluster. It maintains a list of nodes and the Fly.io region where they are deployed in an ETS table that other processes can use to find and initiate their own RPC calls to.","ref":"Fly.RPC.html","title":"Fly.RPC","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Fly.RPC.html#child_spec/1","title":"Fly.RPC.child_spec/1","type":"function"},{"doc":"Callback implementation for GenServer.handle_continue/2 .","ref":"Fly.RPC.html#handle_continue/2","title":"Fly.RPC.handle_continue/2","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"Fly.RPC.html#init/1","title":"Fly.RPC.init/1","type":"function"},{"doc":"Return if the app instance is running in the primary region or not.","ref":"Fly.RPC.html#is_primary?/0","title":"Fly.RPC.is_primary?/0","type":"function"},{"doc":"Executes a function on the remote node to determine if the RPC API support is available. Support may not exist on the remote node in a &quot;first roll out&quot; scenario.","ref":"Fly.RPC.html#is_rpc_supported?/1","title":"Fly.RPC.is_rpc_supported?/1","type":"function"},{"doc":"Return the configured current region. Reads the FLY_REGION ENV setting available when deployed on the Fly.io platform. When running on a different platform, that ENV value will not be set. Setting the MY_REGION ENV value instructs the node how to identify what &quot;region&quot; it is in. If not set, it returns &quot;local&quot; . The value itself is not important. If the value matches the value for the PRIMARY_REGION then it behaves as though it is the primary.","ref":"Fly.RPC.html#my_region/0","title":"Fly.RPC.my_region/0","type":"function"},{"doc":"Return the configured primary region. Reads and requires an ENV setting for PRIMARY_REGION . If not set, it returns &quot;local&quot; .","ref":"Fly.RPC.html#primary_region/0","title":"Fly.RPC.primary_region/0","type":"function"},{"doc":"Asks a node what Fly region it's running in. Returns :error if RPC is not supported on remote node.","ref":"Fly.RPC.html#region/1","title":"Fly.RPC.region/1","type":"function"},{"doc":"Returns the Elixir OTP nodes registered the region. Reads from a local cache.","ref":"Fly.RPC.html#region_nodes/2","title":"Fly.RPC.region_nodes/2","type":"function"},{"doc":"Executes the function on the remote node and waits for the response. Exits after timeout milliseconds.","ref":"Fly.RPC.html#rpc/3","title":"Fly.RPC.rpc/3","type":"function"},{"doc":"Execute the MFA on a node in the primary region.","ref":"Fly.RPC.html#rpc_primary/2","title":"Fly.RPC.rpc_primary/2","type":"function"},{"doc":"Executes the MFA on an available node in the desired region. If the region is the &quot;primary&quot; region or the &quot;local&quot; region then execute the function immediately. Supports the string name of the region or :primary for the current configured primary region. Otherwise find an available node and select one at random to execute the function. Raises ArgumentError when no available nodes. Example &gt; RPC . rpc_region ( &quot;hkg&quot; , fn -&gt; 1 + 2 end ) 3 &gt; RPC . rpc_region ( &quot;hkg&quot; , { Kernel , :+ , [ 1 , 2 ] } ) 3 &gt; RPC . rpc_region ( :primary , { Kernel , :+ , [ 1 , 2 ] } ) 3","ref":"Fly.RPC.html#rpc_region/3","title":"Fly.RPC.rpc_region/3","type":"function"},{"doc":"","ref":"Fly.RPC.html#start_link/1","title":"Fly.RPC.start_link/1","type":"function"},{"doc":"Fly RPC Helps a clustered Elixir application know what Fly.io region it is deployed in, if that is the primary region, and provides features for executing a function through RPC (Remote Procedure Call) on another node by specifying the region to run it in. It is specifically designed to make it easier to execute code in the &quot;primary&quot; region. This library can be used outside of the Fly.io platform as well. In order to work, the nodes need to be clustered and then set PRIMARY_REGION and MY_REGION ENV values. Everything else works the same. When running on Fly.io , the FLY_REGION ENV value provided by the platform is used for MY_REGION . The &quot;primary&quot; region refers to which region your primary, writeable database lives in. Writes to the primary database should ideally be performed from a server running in, or near, the primary region. Online Documentation","ref":"readme.html","title":"Fly RPC","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding fly_rpc to your list of dependencies in mix.exs : def deps do [ { :fly_rpc , &quot;~&gt; 0.2.1&quot; } ] end Through ENV configuration, you can to tell the app which region is the &quot;primary&quot; region. fly.toml This example configuration says that the Sydney Australia region is the &quot;primary&quot; region. [env] PRIMARY_REGION = &quot;syd&quot; Add Fly.RPC to your application's supervision tree. defmodule MyApp.Application do use Application def start ( _type , _args ) do # ... children = [ # Start the RPC server { Fly.RPC , [ ] } , #... ] # ... end end This starts a GenServer that reaches out to other nodes in the cluster to learn what Fly.io regions they are located in. The GenServer caches those results in a local ETS table for fast access.","ref":"readme.html#installation","title":"Fly RPC - Installation","type":"extras"},{"doc":"The Fly.io platform already provides and ENV value of FLY_REGION which this library accesses and uses as the MY_REGION . When using this library on a platform other than fly.io , you can supply the ENV MY_REGION to identify what &quot;region&quot; the running instance is in. Think of the value as a text label of however you want to identify where it's running. Fly . primary_region ( ) #=&gt; &quot;syd&quot; Fly . my_region ( ) #=&gt; &quot;lax&quot; Fly . is_primary? ( ) #=&gt; false The real benefit comes in using the Fly.RPC module. Fly.RPC . rpc_region ( &quot;hkg&quot; , String , :upcase , [ &quot;fly&quot; ] ) #=&gt; &quot;FLY&quot; Fly.RPC . rpc_region ( Fly . primary_region ( ) , String , :upcase , [ &quot;fly&quot; ] ) #=&gt; &quot;FLY&quot; Fly.RPC . rpc_region ( :primary , String , :upcase , [ &quot;fly&quot; ] ) #=&gt; &quot;FLY&quot; Underneath the call, it's using Node.spawn_link/4 . This spawns a new process on a node in the desired region. Normally, that spawn becomes an asynchronous process and what you get back is a pid . In this case, the call executes on the other node and the caller is blocked until the result is received or the request times out. By blocking the process, this makes it much easier to reason about your application code. The following is a convenience function for performing work on the primary. Fly . rpc_primary ( String , :upcase , [ &quot;fly&quot; ] ) #=&gt; &quot;FLY&quot;","ref":"readme.html#usage","title":"Fly RPC - Usage","type":"extras"},{"doc":"When doing local development, the local and primary regions will be set to &quot;local&quot; by default. However, if we want to simulate running in a non-primary region locally, we can set the MY_REGION and PRIMARY_REGION environment variables explicitly: MY_REGION - You tell the library what region it is running in. PRIMARY_REGION - You tell the library which region is the &quot;primary&quot;. By default, the value &quot;local&quot; is used for the regions. This works perfectly for local development as we are, effectively, the primary anyway.","ref":"readme.html#local-development","title":"Fly RPC - Local Development","type":"extras"},{"doc":"When running locally and we explicitly want to set the regions, the MY_REGION isn't set since the app isn't on Fly.io. Also, the PRIMARY_REGION specified in our fly.toml file isn't referenced. We just need a way to set those values when the application is running locally. I like using direnv to automatically set and load ENV values when I enter specific directories. Using direnv , you can create a file named .envrc in your project directory. Add the following lines: export MY_REGION = xyz export PRIMARY_REGION = xyz This tells the app that it's running in the primary region called &quot;xyz&quot;. It will connect to the database and perform writes directly. Another option is to start you application like this: MY_REGION = xyz PRIMARY_REGION = xyz iex - S mix phx . server You can also create a bash script file named start and have it perform the above command.","ref":"readme.html#explicitly-set-the-region","title":"Fly RPC - Explicitly Set the Region","type":"extras"},{"doc":"Prevent temporary outages during deployments When deploying on Fly.io , a new instance is rolled out before removing the old instance. This creates a period of time where both new and old instances are deployed together. By default, when deploying a Phoenix application, a new BEAM cookie is generated for each deployment. When the new instance rolls out with a new BEAM cookie, the old and new instances will not cluster together. BEAM instances must have the same cookie in order to connect. This is by design. This means a newly deployed application running in a secondary region using fly_postgres is unable to perform writes to the older application running in the primary region. It is possible for writes to fail during that rollout window. To prevent this problem, the BEAM cookie can be explicitly set instead of using a randomly generated one for new builds. When explicitly set, the newly deployed application is still able to connect and cluster with the older application running in the primary region. Here is a guide to setting a static cookie for your project that is written into the code itself. This is fine to do because the cookie isn't considered a secret used for security. fly.io/docs/app-guides/elixir-static-cookie/ When the cookie is static and unchanged from one deployment to the next, then applications can continue to cluster and access the applications running in primary region. Where Did My Function Go? When deploying on Fly.io , a new instance is rolled out before removing the old instance. This creates a period of time where both new and old instances are deployed together. In this scenario, let's assume: Node A is an old node. Node B is a new node. Node A attempts to execute a function on Node B. Node B contains new code and that function doesn't exist as called. Perhaps the function was renamed, the arity changed, a pattern match changed or it's a new function. Whatever the reason, we now have a situation where the function we want to call fails because it doesn't exist the way we expect it on the new node in the cluster. Be aware that this can happen. It may cause a single request to fail and that may be fine. We can take steps in our applications to avoid any interruption if it's a critical function that needs to change. For critical functions, the following pattern can be used: Create the new, changed function Maintain backward compatibility with another function (if applicable) Deploy the new code moving all instances to use the new, desired function. During the deploy, the backward compatible function prevent breakage. A later deploy removes the backward compatible function.","ref":"readme.html#production-environment","title":"Fly RPC - Production Environment","type":"extras"},{"doc":"[ ] Instrument with telemetry","ref":"readme.html#features","title":"Fly RPC - Features","type":"extras"}]